package org.apache.bcel.classfile;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.DataInput;

public final class ConstantValue extends Attribute
{
    private int constantValueIndex;
    
    public ConstantValue(final ConstantValue c) {
        this(c.getNameIndex(), c.getLength(), c.getConstantValueIndex(), c.getConstantPool());
    }
    
    ConstantValue(final int nameIndex, final int length, final DataInput input, final ConstantPool constantPool) throws IOException {
        this(nameIndex, length, input.readUnsignedShort(), constantPool);
    }
    
    public ConstantValue(final int nameIndex, final int length, final int constantValueIndex, final ConstantPool constantPool) {
        super((byte)1, nameIndex, length, constantPool);
        this.constantValueIndex = constantValueIndex;
    }
    
    @Override
    public void accept(final Visitor v) {
        v.visitConstantValue(this);
    }
    
    @Override
    public Attribute copy(final ConstantPool constantPool) {
        final ConstantValue c = (ConstantValue)this.clone();
        c.setConstantPool(constantPool);
        return c;
    }
    
    @Override
    public void dump(final DataOutputStream file) throws IOException {
        super.dump(file);
        file.writeShort(this.constantValueIndex);
    }
    
    public int getConstantValueIndex() {
        return this.constantValueIndex;
    }
    
    public void setConstantValueIndex(final int constantValueIndex) {
        this.constantValueIndex = constantValueIndex;
    }
    
    @Override
    public String toString() {
        Constant c = super.getConstantPool().getConstant(this.constantValueIndex);
        String buf = null;
        switch (c.getTag()) {
            case 5: {
                buf = String.valueOf(((ConstantLong)c).getBytes());
                break;
            }
            case 4: {
                buf = String.valueOf(((ConstantFloat)c).getBytes());
                break;
            }
            case 6: {
                buf = String.valueOf(((ConstantDouble)c).getBytes());
                break;
            }
            case 3: {
                buf = String.valueOf(((ConstantInteger)c).getBytes());
                break;
            }
            case 8: {
                final int i = ((ConstantString)c).getStringIndex();
                c = super.getConstantPool().getConstantUtf8(i);
                buf = "\"" + Utility.convertString(((ConstantUtf8)c).getBytes()) + "\"";
                break;
            }
            default: {
                throw new IllegalStateException("Type of ConstValue invalid: " + c);
            }
        }
        return buf;
    }
}
